@@ -581,14 +581,14 @@ private:
         ([]() -> dbus::DBusStruct<uint32_t, uint32_t> {
             return {0, 0};
         }),
-        ([&](dbus::DBusStruct<uint32_t, uint32_t> type) {
+        ([this](dbus::DBusStruct<uint32_t, uint32_t> type) {
             setContentType(std::get<0>(type), std::get<1>(type));
         }),
         dbus::PropertyOption::Hidden);
     FCITX_OBJECT_VTABLE_WRITABLE_PROPERTY(
         clientCommitPreedit, "ClientCommitPreedit", "(b)",
         ([this]() -> dbus::DBusStruct<bool> { return {clientCommitPreedit_}; }),
-        ([&](dbus::DBusStruct<bool> value) {
+        ([this](dbus::DBusStruct<bool> value) {
             clientCommitPreedit_ = std::get<0>(value);
         }),
         dbus::PropertyOption::Hidden);
@@ -695,7 +695,15 @@ IBusFrontendModule::IBusFrontendModule(Instance *instance)
         FCITX_IBUS_WARN() << "Can not get portal ibus name right now.";
     }
 
-    replaceIBus(/*recheck=*/true);
+    // Add some delay for the initial check, since xcb may trigger a ibus
+    // restart on GNOME.
+    timeEvent_ = instance->eventLoop().addTimeEvent(
+        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 1000000, 0,
+        [this](EventSourceTime *, uint64_t) {
+            replaceIBus(/*recheck=*/true);
+            return true;
+        });
+    ;
 }
 
 IBusFrontendModule::~IBusFrontendModule() {
